/* eslint-disable @typescript-eslint/no-var-requires */
// 这里webpack loader 使用 commonjs 语法
const path = require('path')
const babel = require('@babel/core')
const { smart } = require('@babel/template')

module.exports = function (source) {
  const callback = this.async()
  import('@thing.js/cli-utils').then((mod) => {
    const { log, docParser } = mod
    const { resourcePath } = this
    const props = docParser.parse(source)

    const ast = babel.parseSync(source, {
      root: path.resolve(__dirname, '.'),
    })

    // 这里还不能在 entryClass.prototype 上增加 props，会把父类上属性覆盖掉
    // 直接在类里新增 props
    const propsAst = smart.ast(`props =  ${JSON.stringify(props)} ;`)

    // 找到继承自 BasePlugin 的 类
    let entryClassName = ''

    babel.traverse(ast, {
      ClassDeclaration(p) {
        if (
          p.node.superClass?.object?.name === 'THING' &&
          p.node.superClass?.property?.name === 'BasePlugin'
        ) {
          entryClassName = p.node.id?.name

          p.get('body').unshiftContainer('body', propsAst)
        }
      },
    })

    if (!entryClassName) {
      const pluginName = path.basename(path.dirname(resourcePath))
      log.warn(
        'cli',
        `插件 ${pluginName}/index.js 文件没有找到一个继承自 THING.BasePlugin 的类, 将忽略生成 props`
      )
      return callback(null, source)
    }

    babel.traverse(ast, {
      ClassDeclaration(p) {
        if (
          p.node.superClass?.object?.name === 'THING' &&
          p.node.superClass?.property?.name === 'BasePlugin'
        ) {
          entryClassName = p.node.id?.name
        }
      },
    })

    let result

    if (ast) {
      result = babel.transformFromAstSync(ast, '', {
        root: path.resolve(__dirname, '.'),
      })
    }

    callback(null, result?.code)
  })
}
